// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: CoreSDLC wrapper logic instantiates the following sub-blocks:
//				CORESDLC_MAIN, CORESDLC_DUALRAM
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 02June08  	Revision:3.0
//
// SVN Revision Information:
// SVN $Revision: 13653 $
// SVN $Date: 2010-07-08 15:48:23 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4"
//
// *********************************************************************/
`timescale 1ns / 1ps
module CORESDLC (
input PCLK,                                  
input PRESETN,   
input  RXD, 
input  RXC, 
output wire TXD,
input  TXC, 
output wire DEN, 
output wire TV,  
output wire RE,  
output wire RV,  
output wire RDN, 
output wire PTV, 
output wire PRE, 
output wire PRV, 
output wire TDN, 
input [7:0] PWDATA,
input [5:0] PADDR,
output wire [7:0] PRDATA,
input PSEL,
input PENABLE,
input PWRITE,
output wire PREADY,
output wire PSLVERR
);
wire[7:0] p_data_in;
wire[6:0] p_addr_bus;
wire p_we;
wire p_re;
wire[7:0] p_data_out;
wire[7:0] tdib; 
wire[1:0] taddra; 
wire[1:0] taddrb; 
wire twea; 
wire tenb; 
wire[7:0] tdoa; 
wire[7:0] rdib; 
wire[1:0] raddra; 
wire[1:0] raddrb; 
wire rwea; 
wire renb; 
wire[7:0] rdoa; 
assign p_we = (PWRITE && PSEL && PENABLE);
assign p_re = (~PWRITE && PSEL && PENABLE);
assign p_data_in = PWDATA;
assign PRDATA = p_data_out;
assign p_addr_bus = {1'b0,PADDR};
assign PREADY = 1'b1;
assign PSLVERR = 1'b0;
CORESDLC_MAIN u_sdlc (
	.clk(PCLK),
	.nreset(PRESETN),
	.rxd(RXD),
	.txc(TXC),
	.rxc(RXC),
	.tv(TV),
	.re(RE),
	.rv(RV),
	.ptv(PTV),
	.pre(PRE),
	.prv(PRV),
	.rdn(RDN),
	.txd(TXD),
	.den(DEN),
	.p_data_in(p_data_in),
	.p_addr_bus(p_addr_bus),
	.p_we(p_we),
	.p_re(p_re),
	.p_data_out(p_data_out),
	.tdib(tdib),
	.taddra(taddra),
	.taddrb(taddrb),
	.twea(twea),
	.tenb(tenb),
	.tdoa(tdoa),
	.rdib(rdib),
	.raddra(raddra),
	.raddrb(raddrb),
	.rwea(rwea),
	.renb(renb),
	.rdoa(rdoa),
	.tdn(TDN)
);
CORESDLC_DUALRAM u_tx_ram (
	.clkw(PCLK),
	.clkr(PCLK),
	.nreset(PRESETN),
	.we(twea),
	.re(tenb),
	.wdata(tdoa),
	.waddr(taddra),
	.raddr(taddrb),
	.rdata(tdib)
);
CORESDLC_DUALRAM u_rx_ram (
	.clkw(PCLK),
	.clkr(PCLK),
	.nreset(PRESETN),
	.we(rwea),
	.re(renb),
	.wdata(rdoa),
	.waddr(raddra),
	.raddr(raddrb),
	.rdata(rdib)
);
endmodule
