// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: CRC generation/detection logic for CoreSDLC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13668 $
// SVN $Date: 2010-07-08 19:39:16 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_CRC (
input clk, 
input nreset, 
input ct, 
input set, 
input gen, 
input shift, 
input datai, 
output reg datao, 
output reg match
);
reg[31:0] crc; 
reg xor0;
reg xor1;
reg xor2;
reg xor4;
reg xor5;
reg xor7;
reg xor8;
reg xor10;
reg xor11;
reg xor12;
reg xor16;
reg xor22;
reg xor23;
reg xor26;
always @(ct or crc or datai or xor0)
begin
    if (ct == 1'b0)
    begin
        xor0 <= crc[15] ^ datai ;
    end
    else
    begin
        xor0 <= crc[31] ^ datai ;
    end
    xor1 <= xor0 ^ crc[0] ;
    xor2 <= xor0 ^ crc[1] ;
    xor4 <= xor0 ^ crc[3] ;
    xor5 <= xor0 ^ crc[4] ;
    xor7 <= xor0 ^ crc[6] ;
    xor8 <= xor0 ^ crc[7] ;
    xor10 <= xor0 ^ crc[9] ;
    xor11 <= xor0 ^ crc[10] ;
    xor12 <= xor0 ^ crc[11] ;
    xor16 <= xor0 ^ crc[15] ;
    xor22 <= xor0 ^ crc[21] ;
    xor23 <= xor0 ^ crc[22] ;
    xor26 <= xor0 ^ crc[25] ;
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        crc <= 32'hffffffff;
    end
    else
    begin
        if (set == 1'b1)
        begin
            crc <= 32'hffffffff;
        end
        else if (gen == 1'b1)
        begin
            case (ct)
                1'b0 :
                begin
                    crc <= {crc[30:15], crc[14:12], xor12,
                    crc[10:5], xor5, crc[3:0], xor0} ;
                end
                default :
                begin
                    crc <= {crc[30:26], xor26, crc[24:23],
                    xor23, xor22, crc[20:16], xor16, crc[14:12]
                    , xor12, xor11, xor10, crc[8], xor8, xor7,
                    crc[5], xor5, xor4, crc[2], xor2, xor1,
                    xor0} ;
                end
            endcase
        end
        else if (shift == 1'b1)
        begin
            crc <= {crc[30:0], 1'b1} ; 
        end
    end
end
always @(posedge clk or negedge nreset)
begin : output_proc
    if (nreset == 1'b0)
    begin
        datao <= 1'b0 ;
    end
    else
    begin
        if (ct == 1'b0)
        begin
            datao <= ~crc[15] ;
        end
        else
        begin
            datao <= ~crc[31] ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin : crc_match_proc
    if (nreset == 1'b0)
    begin
        match <= 1'b1 ;
    end
    else
    begin
        case (ct)
            1'b0 :
            begin
                if (crc[15:0] == 16'b0001_1101_0000_1111)
                begin
                    match <= 1'b1 ;
                end
                else
                begin
                    match <= 1'b0 ;
                end
            end
            default :
            begin
                if (crc[31:0] == 32'b11000111_00000100_11011101_01111011)
                begin
                    match <= 1'b1 ;
                end
                else
                begin
                    match <= 1'b0 ;
                end
            end
        endcase
    end
end
endmodule
