// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: parameterized pseudo dual-port (read data out only) RAM
//				with synch. act.high read & write signals, and read data out
//				is transparent, i.e., available on rising edge of read
//				clock (not pipelined)
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13654 $
// SVN $Date: 2010-07-08 15:49:19 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_DUALRAM
#(
parameter WIDTH  = 8,
parameter DEPTH  = 3
) (
input clkw,
input clkr,
input nreset,
input we,
input re,
input[WIDTH - 1:0] wdata,
input[1:0] waddr,
input[1:0] raddr,
output wire[WIDTH - 1:0] rdata
);
reg[WIDTH - 1:0] store[0:DEPTH - 1];
reg[WIDTH - 1:0] rdata_int;
always @(posedge clkw or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        begin : store_reset
            integer i;
            for(i = 0; i <= DEPTH - 1; i = i + 1)
            begin
                store[i] <= {WIDTH{1'b0}} ;
            end
        end
    end
    else
    begin
        if (we == 1'b1)
        begin
            store[waddr] <= wdata ;
        end
    end
end
always @(posedge clkr or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rdata_int <= {WIDTH{1'b0}} ;
    end
    else
    begin
        if (re == 1'b1)
        begin
            rdata_int <= store[raddr] ;
        end
    end
end
assign rdata = rdata_int ;
endmodule
