// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: Main CoreSDLC logic (except for Rx/Tx FIFOs)
//				instantiates the following sub-blocks:
//				CORESDLC_RECEIVE, CORESDLC_TRANSMIT, CORESDLC_SFR
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13653 $
// SVN $Date: 2010-07-08 15:48:23 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_MAIN (
input clk, 
input nreset, 
input rxd, 
input txc, 
input rxc, 
output wire tdn, 
output wire txd, 
output wire den, 
output wire tv, 
output wire re, 
output wire rv, 
output wire rdn, 
output wire ptv, 
output wire pre, 
output wire prv, 
input [7:0] p_data_in, 
input [6:0] p_addr_bus, 
input p_we, 
input p_re, 
output wire [7:0] p_data_out, 
input [7:0] tdib, 
output wire [1:0] taddra, 
output wire [1:0] taddrb, 
output wire twea,
output wire tenb,
output wire [7:0] tdoa,
input [7:0] rdib, 
output wire [1:0] raddra,
output wire [1:0] raddrb,
output wire rwea,
output wire renb,
output wire [7:0] rdoa
);
wire[7:0] ifs; 
wire[7:0] ifsreg; 
wire[7:0] tfifo; 
wire[7:0] rfifo; 
wire[7:0] adr0; 
wire[7:0] adr1; 
wire[7:0] adr2; 
wire[7:0] adr3; 
wire[7:0] amsk0; 
wire[7:0] amsk1; 
wire bce; 
wire al; 
wire rfiford; 
wire rfne; 
wire irdn; 
wire gren; 
wire grenset; 
wire crce; 
wire ae; 
wire rcabt; 
wire ovr; 
wire lni; 
wire garen; 
wire xrclk; 
wire tfifowr; 
wire ten; 
wire tenset; 
wire tfnf; 
wire[1:0] pl; 
wire xtclk; 
wire txdi; 
wire gfien; 
wire ct; 
wire[1:0] mode; 
CORESDLC_SFR u_sfr (
	.clk(clk),
	.nreset(nreset),
	.sfrdatai(p_data_in),
	.sfraddr(p_addr_bus),
	.sfrwe(p_we),
	.sfrra(p_re),
	.sfrdatao(p_data_out),
	.tv(tv),
	.re(re),
	.rv(rv),
	.ptv(ptv),
	.pre(pre),
	.prv(prv),
	.al(al),
	.adr0(adr0),
	.adr1(adr1),
	.adr2(adr2),
	.adr3(adr3),
	.amsk0(amsk0),
	.amsk1(amsk1),
	.gren(gren),
	.grenset(grenset),
	.garen(garen),
	.rfiford(rfiford),
	.rfifo(rfifo),
	.rfne(rfne),
	.rdn(irdn),
	.crce(crce),
	.ae(ae),
	.rcabt(rcabt),
	.ovr(ovr),
	.lni(lni),
	.xrclk(xrclk),
	.pl(pl),
	.ifsreg(ifsreg),
	.ten(ten),
	.tenset(tenset),
	.gfien(gfien),
	.tfifowr(tfifowr),
	.tfifo(tfifo),
	.tfnf(tfnf),
	.tdn(tdn),
	.ifs(ifs),
	.xtclk(xtclk),
	.bce(bce),
	.ct(ct),
	.mode(mode)
);
CORESDLC_RECEIVE u_receive (
	.clk(clk),
	.nreset(nreset),
	.bce(bce),
	.rxc(rxc),
	.xrclk(xrclk),
	.rxd(rxd),
	.txd(txdi),
	.ct(ct),
	.al(al),
	.mode(mode),
	.adr0(adr0),
	.adr1(adr1),
	.adr2(adr2),
	.adr3(adr3),
	.amsk0(amsk0),
	.amsk1(amsk1),
	.gren(gren),
	.grenset(grenset),
	.garen(garen),
	.rfiford(rfiford),
	.rfifo(rfifo),
	.rfne(rfne),
	.rdn(irdn),
	.crce(crce),
	.ae(ae),
	.rcabt(rcabt),
	.ovr(ovr),
	.lni(lni),
	.dib(rdib),
	.addra(raddra),
	.addrb(raddrb),
	.wea(rwea),
	.enb(renb),
	.doa(rdoa)
);
CORESDLC_TRANSMIT u_transmit (
	.clk(clk),
	.nreset(nreset),
	.bce(bce),
	.txc(txc),
	.xtclk(xtclk),
	.ct(ct),
	.mode(mode),
	.pl(pl),
	.gfien(gfien),
	.ifsreg(ifsreg),
	.ten(ten),
	.tenset(tenset),
	.tfifowr(tfifowr),
	.tfifo(p_data_in),
	.tfnf(tfnf),
	.tdn(tdn),
	.ifs(ifs),
	.txd(txdi),
	.den(den),
	.dib(tdib),
	.addra(taddra),
	.addrb(taddrb),
	.wea(twea),
	.enb(tenb),
	.doa(tdoa)
);
assign txd = txdi ;
assign rdn = irdn ;
endmodule
