// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: CoreSDLC Receive logic instantiates the following sub-blocks:
//				CORESDLC_FIFO, CORESDLC_CRC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13659 $
// SVN $Date: 2010-07-08 19:01:18 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_RECEIVE (
input clk, 
input nreset, 
input bce, 
input rxc, 
input xrclk, 
input rxd, 
input txd, 
input ct, 
input al, 
input [1:0] mode, 
input [7:0] adr0, 
input [7:0] adr1, 
input [7:0] adr2, 
input [7:0] adr3, 
input [7:0] amsk0, 
input [7:0] amsk1, 
input gren, 
input grenset, 
input garen, 
input rfiford, 
output wire [7:0] rfifo, 
output wire rfne, 
output reg rdn, 
output reg crce, 
output reg ae, 
output reg rcabt, 
output reg ovr, 
output reg lni, 
input [7:0] dib,
output wire [1:0] addra,
output wire [1:0] addrb,
output wire wea,
output wire enb,
output wire [7:0] doa
);
localparam[1:0] RT_MODE = 2'b01;
localparam[1:0] RR_MODE = 2'b10;
localparam[2:0] SM_IDLE = 0;
localparam[2:0] SM_ADDRESS = 1;
localparam[2:0] SM_WAIT = 2;
localparam[2:0] SM_DATA = 3;
localparam[2:0] SM_CRC = 4;
localparam[2:0] CSM_STABLE = 0;
localparam[2:0] CSM_EDGE = 1;
localparam[2:0] CSM_EDGELATE = 2;
localparam[2:0] CSM_ONE = 3;
localparam[2:0] CSM_ZERO = 4;
reg fifowr; 
wire empty; 
wire full; 
reg crcset; 
reg crcgen; 
wire crcshift; 
reg crcin; 
wire crcout; 
wire crcmatch; 
reg stripping; 
reg[2:0] strip_count; 
reg[2:0] sm_nxt; 
reg[2:0] sm; 
reg[2:0] bit_count; 
reg[1:0] byte_count; 
reg greni; 
reg[7:0] adr_lsb; 
reg[7:0] adr_msb; 
reg adr0_match; 
reg adr1_match; 
reg adr2_match; 
reg adr3_match; 
reg broadcast_match; 
reg adr_match; 
reg[3:0] flag_count; 
reg bof_det; 
reg abort_det; 
reg zero_det; 
reg[31:0] delay; 
reg[7:0] data; 
reg data_in; 
reg rxd_ff; 
reg rd; 
reg rd_prev; 
reg rdata; 
reg[2:0] csm_nxt; 
reg[2:0] csm; 
reg rxc_ff; 
reg rxc_ff2; 
reg[2:0] rcount; 
reg rce; 
reg rce_ff; 
reg rce_ff2; 
reg[1:0] edge_count; 
CORESDLC_FIFO u_fifo (
	.nreset(nreset),
	.clk(clk),
	.datai(data),
	.rd(rfiford),
	.wr(fifowr),
	.clear(grenset),
	.datao(rfifo),
	.empty(empty),
	.full(full),
	.dib(dib),
	.addra(addra),
	.addrb(addrb),
	.wea(wea),
	.enb(enb),
	.doa(doa)
);
assign crcshift = 1'b0 ;
CORESDLC_CRC u_crc (
	.clk(clk),
	.nreset(nreset),
	.ct(ct),
	.set(crcset),
	.gen(crcgen),
	.shift(crcshift),
	.datai(crcin),
	.datao(crcout),
	.match(crcmatch)
);
assign rfne = ~empty ;
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        flag_count <= 4'h0;
        bof_det <= 1'b0 ;
        abort_det <= 1'b0 ;
        lni <= 1'b0 ;
        zero_det <= 1'b0 ;
    end
    else
    begin
        if (rce == 1'b1)
        begin
            if (rdata == 1'b0)
            begin
                flag_count <= 4'h0;
                zero_det <= 1'b1 ;
            end
            else
            begin
                if (flag_count != 4'b1110)
                begin
                    flag_count <= flag_count + 1'b1 ;
                end
            end
        end
        if (rdata == 1'b0 & flag_count == 4'b0110 & zero_det == 1'b1)
        begin
            bof_det <= 1'b1 ;
        end
        else
        begin
            bof_det <= 1'b0 ;
        end
        if (rdata == 1'b1 & flag_count == 4'b0110)
        begin
            abort_det <= 1'b1 ;
        end
        else
        begin
            abort_det <= 1'b0 ;
        end
        if (rdata == 1'b1 & flag_count == 4'b1110)
        begin
            lni <= 1'b1 ;
        end
        else
        begin
            lni <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        greni <= 1'b0 ;
    end
    else
    begin
        if ((gren == 1'b0) & (garen == 1'b0))
        begin
            greni <= 1'b0 ;
        end
        else
        begin
            if (rce_ff == 1'b1)
            begin
                greni <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        sm <= SM_IDLE ;
    end
    else
    begin
        if (rce_ff == 1'b1 & stripping == 1'b0)
        begin
            sm <= sm_nxt ;
        end
    end
end
always @(sm or garen or gren or greni or full or mode or bof_det or
abort_det or bit_count or byte_count or adr_match)
begin
    case (sm)
        SM_IDLE :
        begin
            if ((garen == 1'b1 | gren == 1'b1) & bof_det == 1'b1)
            begin
                if (mode == RR_MODE)
                begin
                    sm_nxt <= SM_WAIT ;
                end
                else
                begin
                    sm_nxt <= SM_ADDRESS ;
                end
            end
            else
            begin
                sm_nxt <= SM_IDLE ;
            end
        end
        SM_ADDRESS :
        begin
            if (greni == 1'b0)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else if (bof_det == 1'b1)
            begin
                sm_nxt <= SM_ADDRESS ;
            end
            else if (abort_det == 1'b1)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else
            begin
                if (bit_count == 3'b000 & byte_count == 2'b00)
                begin
                    if (adr_match == 1'b1)
                    begin
                        sm_nxt <= SM_WAIT ;
                    end
                    else
                    begin
                        sm_nxt <= SM_IDLE ;
                    end
                end
                else
                begin
                    sm_nxt <= SM_ADDRESS ;
                end
            end
        end
        SM_WAIT :
        begin
            if (greni == 1'b0)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else if (bof_det == 1'b1)
            begin
                if (mode == RR_MODE)
                begin
                    sm_nxt <= SM_WAIT ;
                end
                else
                begin
                    sm_nxt <= SM_ADDRESS ;
                end
            end
            else if (abort_det == 1'b1)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else
            begin
                if (bit_count == 3'b000 & byte_count == 2'b00)
                begin
                    sm_nxt <= SM_DATA ;
                end
                else
                begin
                    sm_nxt <= SM_WAIT ;
                end
            end
        end
        default :
        begin
            if (greni == 1'b0)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else if (bof_det == 1'b1)
            begin
                if (garen == 1'b1)
                begin
                    if (mode == RR_MODE)
                    begin
                        sm_nxt <= SM_WAIT ;
                    end
                    else
                    begin
                        sm_nxt <= SM_ADDRESS ;
                    end
                end
                else
                begin
                    sm_nxt <= SM_IDLE ;
                end
            end
            else if (abort_det == 1'b1)
            begin
                sm_nxt <= SM_IDLE ;
            end
            else
            begin
                if (bit_count == 3'b000 & full == 1'b1)
                begin
                    sm_nxt <= SM_IDLE ;
                end
                else
                begin
                    sm_nxt <= SM_DATA ;
                end
            end
        end
    endcase
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        crcset <= 1'b0 ;
    end
    else
    begin
        if (sm == SM_ADDRESS & (al == 1'b0 | byte_count == 2'b01) &
        bit_count != 3'b000)
        begin
            crcset <= 1'b1 ;
        end
        else
        begin
            crcset <= 1'b0 ;
        end
    end
end
always @(rce_ff or stripping)
begin
    if (rce_ff == 1'b1 & stripping == 1'b0)
    begin
        crcgen <= 1'b1 ;
    end
    else
    begin
        crcgen <= 1'b0 ;
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        fifowr <= 1'b0 ;
    end
    else
    begin
        if (sm == SM_DATA & bit_count == 3'b111 & rce_ff2 == 1'b1 &
        stripping == 1'b0)
        begin
            fifowr <= 1'b1 ;
        end
        else
        begin
            fifowr <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        bit_count <= 3'b111;
    end
    else
    begin
        if (rce_ff == 1'b1 & stripping == 1'b0)
        begin
            if (bof_det == 1'b1)
            begin
                bit_count <= 3'b111;
            end
            else
            begin
                bit_count <= bit_count - 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        byte_count <= 2'b11;
    end
    else
    begin
        if (rce_ff == 1'b1 & stripping == 1'b0)
        begin
            if (bof_det == 1'b1)
            begin
                if (mode == RR_MODE)
                begin
                    byte_count <= 2'b00 ;
                end
                else
                begin
                    if (al == 1'b0)
                    begin
                        byte_count <= 2'b00 ;
                    end
                    else
                    begin
                        byte_count <= 2'b01 ;
                    end
                end
            end
            else if (sm == SM_ADDRESS & bit_count == 3'b000 & byte_count
            == 2'b00)
            begin
                if (al == 1'b0 & ct == 1'b0)
                begin
                    byte_count <= 2'b01 ;
                end
                else if (al == 1'b0 & ct == 1'b1)
                begin
                    byte_count <= 2'b11 ;
                end
                else if (al == 1'b1 & ct == 1'b0)
                begin
                    byte_count <= 2'b00 ;
                end
                else
                begin
                    byte_count <= 2'b10 ;
                end
            end
            else
            begin
                if (bit_count == 3'b000)
                begin
                    byte_count <= byte_count - 1'b1 ;
                end
            end
        end
    end
end
always @(sm or adr0 or adr1 or adr2 or adr3 or rdata or amsk0 or amsk1 or
         adr_msb or adr_lsb or al or delay)
begin
    if (sm == SM_ADDRESS)
    begin
        if (adr_msb == 8'b11111111 & adr_lsb == 8'b11111111)
        begin
            broadcast_match <= 1'b1 ;
        end
        else
        begin
            broadcast_match <= 1'b0 ;
        end
    end
    else
    begin
        broadcast_match <= 1'b0 ;
    end
    if (sm == SM_ADDRESS)
    begin
        if ((~(adr_lsb ^ adr0) | amsk0) == 8'b11111111)
        begin
            adr0_match <= 1'b1 ;
        end
        else
        begin
            adr0_match <= 1'b0 ;
        end
    end
    else
    begin
        adr0_match <= 1'b0 ;
    end
    if (sm == SM_ADDRESS)
    begin
        if ((~(adr_msb ^ adr1) | amsk1) == 8'b11111111)
        begin
            adr1_match <= 1'b1 ;
        end
        else
        begin
            adr1_match <= 1'b0 ;
        end
    end
    else
    begin
        adr1_match <= 1'b0 ;
    end
    if (sm == SM_ADDRESS)
    begin
        if (adr_lsb == adr2)
        begin
            adr2_match <= 1'b1 ;
        end
        else
        begin
            adr2_match <= 1'b0 ;
        end
    end
    else
    begin
        adr2_match <= 1'b0 ;
    end
    if (sm == SM_ADDRESS)
    begin
        if (adr_msb == adr3)
        begin
            adr3_match <= 1'b1 ;
        end
        else
        begin
            adr3_match <= 1'b0 ;
        end
    end
    else
    begin
        adr3_match <= 1'b0 ;
    end
    if (al == 1'b0)
    begin
        adr_lsb <= {rdata, delay[31:25]} ;
        adr_msb <= {rdata, delay[31:25]} ;
    end
    else
    begin
        adr_msb <= {rdata, delay[31:25]} ;
        adr_lsb <= delay[24:17] ;
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        adr_match <= 1'b0 ;
    end
    else
    begin
        case (al)
            1'b0 :
            begin
                if (broadcast_match == 1'b1 | adr0_match == 1'b1 |
                adr1_match == 1'b1 | adr2_match == 1'b1 | adr3_match == 1'b1)
                begin
                    adr_match <= 1'b1 ;
                end
                else
                begin
                    adr_match <= 1'b0 ;
                end
            end
            default :
            begin
                if (broadcast_match == 1'b1 |
               (adr0_match == 1'b1 & adr1_match == 1'b1) |
               (adr2_match == 1'b1 & adr3_match == 1'b1))
                begin
                    adr_match <= 1'b1 ;
                end
                else
                begin
                    adr_match <= 1'b0 ;
                end
            end
        endcase
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        strip_count <= 3'b101 ;
        stripping <= 1'b0 ;
    end
    else
    begin
        if (rce == 1'b1)
        begin
            if (stripping == 1'b1 | rdata == 1'b0)
            begin
                strip_count <= 3'b101 ;
            end
            else
            begin
                strip_count <= strip_count - 1'b1 ;
            end
        end
        if (strip_count == 3'b000 & rdata == 1'b0)
        begin
            stripping <= 1'b1 ;
        end
        else
        begin
            stripping <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        delay <= 32'h00000000;
    end
    else
    begin
        if (rce_ff == 1'b1 & stripping == 1'b0)
        begin
            delay <= {rdata, delay[31:1]} ;
        end
    end
end
always @(delay or ct or rdata or mode)
begin
    if (mode == RR_MODE)
    begin
        data_in <= rdata ;
    end
    else
    begin
        if (ct == 1'b0)
        begin
            data_in <= delay[16] ;
        end
        else
        begin
            data_in <= delay[0] ;
        end
    end
    crcin <= delay[25] ;
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        data <= 8'h00;
    end
    else
    begin
        if (rce_ff == 1'b1 & stripping == 1'b0)
        begin
            data <= {data_in, data[7:1]} ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rdn <= 1'b0 ;
    end
    else
    begin
        if (grenset == 1'b1)
        begin
            rdn <= 1'b0 ;
        end
        else
        begin
            if (((sm == SM_DATA) & (bof_det == 1'b1) &
            (crcmatch == 1'b1 | mode == RR_MODE) & (gren == 1'b1) &
            (rce_ff == 1'b1)) & bit_count == 3'b000)
            begin
                rdn <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        crce <= 1'b0 ;
    end
    else
    begin
        if (grenset == 1'b1)
        begin
            crce <= 1'b0 ;
        end
        else
        begin
            if (((bof_det == 1'b1 | abort_det == 1'b1) |
            (bit_count == 3'b000 & full == 1'b1)) & crcmatch == 1'b0 &
            mode != RR_MODE & rce_ff == 1'b1 & sm == SM_DATA)
            begin
                crce <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ae <= 1'b0 ;
    end
    else
    begin
        if (grenset == 1'b1)
        begin
            ae <= 1'b0 ;
        end
        else
        begin
            if ((sm == SM_DATA) & (bof_det == 1'b1 | abort_det == 1'b1) &
            (bit_count != 3'b000) & (rce_ff == 1'b1))
            begin
                ae <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rcabt <= 1'b0 ;
    end
    else
    begin
        if (grenset == 1'b1)
        begin
            rcabt <= 1'b0 ;
        end
        else
        begin
            if (sm == SM_DATA & abort_det == 1'b1 & rce_ff == 1'b1)
            begin
                rcabt <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ovr <= 1'b0 ;
    end
    else
    begin
        if (grenset == 1'b1)
        begin
            ovr <= 1'b0 ;
        end
        else
        begin
            if (sm == SM_DATA & bit_count == 3'b000 & full == 1'b1 &
            rce_ff == 1'b1)
            begin
                ovr <= 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rdata <= 1'b1 ;
    end
    else
    begin
        if (rce == 1'b1)
        begin
            if (xrclk == 1'b1)
            begin
                rdata <= rd ;
            end
            else
            begin
                if (csm == CSM_ZERO)
                begin
                    rdata <= 1'b0 ;
                end
                else
                begin
                    rdata <= 1'b1 ;
                end
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        csm <= CSM_STABLE ;
    end
    else
    begin
        if (bce == 1'b1)
        begin
            csm <= csm_nxt ;
        end
    end
end
always @(csm or rd or rd_prev or rcount or edge_count)
begin
    case (csm)
        CSM_STABLE :
        begin
            if (rd != rd_prev)
            begin
                if (rcount != 3'b000)
                begin
                    csm_nxt <= CSM_EDGE ;
                end
                else
                begin
                    csm_nxt <= CSM_EDGELATE ;
                end
            end
            else
            begin
                if (rcount == 3'b000)
                begin
                    csm_nxt <= CSM_ONE ;
                end
                else
                begin
                    csm_nxt <= CSM_STABLE ;
                end
            end
        end
        CSM_ONE :
        begin
            if (rd == rd_prev)
            begin
                csm_nxt <= CSM_STABLE ;
            end
            else
            begin
                csm_nxt <= CSM_EDGE ;
            end
        end
        CSM_EDGE :
        begin
            if (edge_count == 2'b00)
            begin
                csm_nxt <= CSM_ZERO ;
            end
            else
            begin
                if (rcount == 3'b000)
                begin
                    if (rd != rd_prev)
                    begin
                        csm_nxt <= CSM_ONE ;
                    end
                    else
                    begin
                        csm_nxt <= CSM_EDGELATE ;
                    end
                end
                else
                begin
                    if (rd != rd_prev)
                    begin
                        csm_nxt <= CSM_STABLE ;
                    end
                    else
                    begin
                        csm_nxt <= CSM_EDGE ;
                    end
                end
            end
        end
        CSM_EDGELATE :
        begin
            if (edge_count == 2'b00)
            begin
                csm_nxt <= CSM_ZERO ;
            end
            else
            begin
                if (rd != rd_prev)
                begin
                    csm_nxt <= CSM_ONE ;
                end
                else
                begin
                    csm_nxt <= CSM_EDGELATE ;
                end
            end
        end
        default :
        begin
            csm_nxt <= CSM_STABLE ;
        end
    endcase
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        edge_count <= 2'b10 ;
    end
    else
    begin
        if (csm == CSM_STABLE | csm == CSM_ONE)
        begin
            edge_count <= 2'b11 ;
        end
        else
        begin
            if (bce == 1'b1)
            begin
                edge_count <= edge_count - 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rxc_ff <= 1'b0 ;
        rxc_ff2 <= 1'b0 ;
    end
    else
    begin
        rxc_ff <= rxc ;
        rxc_ff2 <= rxc_ff ;
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rce <= 1'b0 ;
        rce_ff <= 1'b0 ;
        rce_ff2 <= 1'b0 ;
    end
    else
    begin
        case (xrclk)
            1'b0 :
            begin
                if ((bce == 1'b1) & (csm_nxt == CSM_ONE | csm_nxt == CSM_ZERO))
                begin
                    rce <= 1'b1 ;
                    rce_ff <= 1'b1 ;
                end
                else
                begin
                    rce <= 1'b0 ;
                    rce_ff <= 1'b0 ;
                end
                rce_ff2 <= rce_ff ;
            end
            default :
            begin
                if (rxc_ff2 == 1'b0 & rxc_ff == 1'b1)
                begin
                    rce <= 1'b1 ;
                end
                else
                begin
                    rce <= 1'b0 ;
                end
                if (rxc_ff2 == 1'b1 & rxc_ff == 1'b0)
                begin
                    rce_ff <= 1'b1 ;
                end
                else
                begin
                    rce_ff <= 1'b0 ;
                end
                rce_ff2 <= rce_ff ;
            end
        endcase
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rcount <= 3'b111;
    end
    else
    begin
        if (bce == 1'b1)
        begin
            if (csm_nxt == CSM_ZERO)
            begin
                rcount <= 3'b111;
            end
            else
            begin
                rcount <= rcount - 1'b1 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rd <= 1'b0 ;
    end
    else
    begin
        if (mode == RT_MODE)
        begin
            rd <= txd ;
        end
        else
        begin
            rd <= rxd_ff ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rxd_ff <= 1'b1 ;
    end
    else
    begin
        rxd_ff <= rxd ;
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rd_prev <= 1'b1 ;
    end
    else
    begin
        if (bce == 1'b1)
        begin
            rd_prev <= rd ;
        end
    end
end
endmodule
