// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: Special Function Register interface (CPU I/F) for CoreSDLC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13654 $
// SVN $Date: 2010-07-08 15:49:19 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4", all port signals are
//				active high, unless otherwise noted
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_SFR (
input clk, 
input nreset, 
input[7:0] sfrdatai, 
input[6:0] sfraddr, 
input sfrwe, 
input sfrra, 
output wire[7:0] sfrdatao, 
output reg tv, 
output reg re, 
output reg rv, 
output wire ptv, 
output wire pre, 
output wire prv, 
input[7:0] rfifo, 
input rfne, 
input rdn, 
input crce, 
input ae, 
input rcabt, 
input ovr, 
input lni, 
output wire al, 
output wire gren, 
output wire grenset, 
output wire garen, 
output wire rfiford, 
output wire[7:0] adr0, 
output wire[7:0] adr1, 
output wire[7:0] adr2, 
output wire[7:0] adr3, 
output wire[7:0] amsk0, 
output wire[7:0] amsk1, 
output wire xrclk, 
input tfnf, 
input tdn, 
input[7:0] ifs, 
output wire[1:0] pl, 
output reg[7:0] ifsreg, 
output wire ten, 
output wire tenset, 
output wire gfien, 
output wire tfifowr, 
output wire[7:0] tfifo, 
output wire xtclk, 
output reg bce, 
output wire ct, 
output wire[1:0] mode
);
localparam[6:0]	PCON_ID		= 7'h08;
localparam[6:0]	GMOD_ID		= 7'h00;
localparam[6:0]	TFIFO_ID	= 7'h04;
localparam[6:0]	BAUD_ID		= 7'h0C;
localparam[6:0]	ADR0_ID		= 7'h10;
localparam[6:0]	IFS_ID		= 7'h14;
localparam[6:0]	ADR1_ID		= 7'h18;
localparam[6:0]	ADR2_ID		= 7'h1c;
localparam[6:0]	ADR3_ID		= 7'h20;
localparam[6:0]	IEN1_ID		= 7'h24;
localparam[6:0]	AMSK0_ID	= 7'h28;
localparam[6:0]	TSTAT_ID 	= 7'h2C;
localparam[6:0]	AMSK1_ID	= 7'h30;
localparam[6:0]	RSTAT_ID	= 7'h34;
localparam[6:0]	RFIFO_ID	= 7'h38;
localparam[6:0]	IPN1_ID		= 7'h3c;
reg[4:2] pcon42; 
reg[7:1] gmod71; 
reg ien133; 
reg[1:0] ien110; 
reg ipn133; 
reg[1:0] ipn110; 
reg tstat11; 
reg rstat11; 
reg[7:0] adr0_reg; 
reg[7:0] adr1_reg; 
reg[7:0] adr2_reg; 
reg[7:0] adr3_reg; 
reg[7:0] amsk0_reg; 
reg[7:0] amsk1_reg; 
reg[7:0] baud_count; 
reg[7:0] baudreg; 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        pcon42 <= 3'b000;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == PCON_ID)
        begin
            pcon42 <= sfrdatai[4:2] ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        gmod71 <= 7'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == GMOD_ID)
        begin
            gmod71 <= sfrdatai[7:1] ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        baudreg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == BAUD_ID)
        begin
            baudreg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        adr0_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == ADR0_ID)
        begin
            adr0_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ifsreg <= 8'b00000001 ;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == IFS_ID)
        begin
            ifsreg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        adr1_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == ADR1_ID)
        begin
            adr1_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        adr2_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == ADR2_ID)
        begin
            adr2_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        adr3_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == ADR3_ID)
        begin
            adr3_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ien133 <= 1'b0 ;
        ien110 <= 2'b00 ;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == IEN1_ID)
        begin
            ien133 <= sfrdatai[3] ;
            ien110 <= sfrdatai[1:0] ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        amsk0_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == AMSK0_ID)
        begin
            amsk0_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        tstat11 <= 1'b0 ;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == TSTAT_ID)
        begin
            tstat11 <= sfrdatai[1] ;
        end
        else
        begin
            if (tdn == 1'b1)
            begin
                tstat11 <= 1'b0 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        amsk1_reg <= 8'h00;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == AMSK1_ID)
        begin
            amsk1_reg <= sfrdatai ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rstat11 <= 1'b0 ;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == RSTAT_ID)
        begin
            rstat11 <= sfrdatai[1] ;
        end
        else
        begin
            if (rdn == 1'b1 | crce == 1'b1 | ae == 1'b1 | rcabt == 1'b1 |
            ovr == 1'b1)
            begin
                rstat11 <= 1'b0 ;
            end
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ipn133 <= 1'b0 ;
        ipn110 <= 2'b00 ;
    end
    else
    begin
        if (sfrwe == 1'b1 & sfraddr == IPN1_ID)
        begin
            ipn133 <= sfrdatai[3] ;
            ipn110 <= sfrdatai[1:0] ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        tv <= 1'b0 ;
    end
    else
    begin
        if (tfnf == 1'b1 & ien133 == 1'b1)
        begin
            tv <= 1'b1 ;
        end
        else
        begin
            tv <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        rv <= 1'b0 ;
    end
    else
    begin
        if (rfne == 1'b1 & (ien110[0]) == 1'b1)
        begin
            rv <= 1'b1 ;
        end
        else
        begin
            rv <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        re <= 1'b0 ;
    end
    else
    begin
        if ((ovr == 1'b1 | ae == 1'b1 | rcabt == 1'b1 | crce == 1'b1) &
        (ien110[1]) == 1'b1)
        begin
            re <= 1'b1 ;
        end
        else
        begin
            re <= 1'b0 ;
        end
    end
end
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        baud_count <= 8'h00;
        bce <= 1'b0 ;
    end
    else
    begin
        if (baud_count == 8'b00000000)
        begin
            baud_count <= baudreg ;
        end
        else
        begin
            baud_count <= baud_count - 1'b1 ;
        end
        if (baud_count == 8'b00000000)
        begin
            bce <= 1'b1 ;
        end
        else
        begin
            bce <= 1'b0 ;
        end
    end
end
assign ct = gmod71[3] ;
assign al = gmod71[4] ;
assign mode = gmod71[6:5] ;
assign gren = rstat11 ;
assign garen = pcon42[4] ;
assign gfien = pcon42[2] ;
assign xrclk = pcon42[3] ;
assign pl = gmod71[2:1] ;
assign ten = tstat11 ;
assign xtclk = gmod71[7] ;
assign tfifo = sfrdatai ;
assign adr0 = adr0_reg ;
assign adr1 = adr1_reg ;
assign adr2 = adr2_reg ;
assign adr3 = adr3_reg ;
assign amsk0 = amsk0_reg ;
assign amsk1 = amsk1_reg ;
assign ptv = ipn133 ;
assign prv = ipn110[1] ;
assign pre = ipn110[0] ;
assign tfifowr = (sfrwe == 1'b1 & sfraddr == TFIFO_ID) ? 1'b1 : 1'b0 ;
assign rfiford = (sfrra == 1'b1 & sfraddr == RFIFO_ID) ? 1'b1 : 1'b0 ;
assign grenset = (sfrwe == 1'b1 & sfraddr == RSTAT_ID &
 (sfrdatai[1]) == 1'b1) ? 1'b1 : 1'b0 ;
assign tenset = (sfrwe == 1'b1 & sfraddr == TSTAT_ID &
 (sfrdatai[1]) == 1'b1) ? 1'b1 : 1'b0 ;
assign sfrdatao =
 (sfraddr == PCON_ID)  ? {3'b000, pcon42, 2'b00} :
 (sfraddr == GMOD_ID)  ? {gmod71, 1'b0} :
 (sfraddr == BAUD_ID)  ? baud_count :
 (sfraddr == ADR0_ID)  ? adr0_reg :
 (sfraddr == IFS_ID)   ? ifs :
 (sfraddr == ADR1_ID)  ? adr1_reg :
 (sfraddr == ADR2_ID)  ? adr2_reg :
 (sfraddr == ADR3_ID)  ? adr3_reg :
 (sfraddr == IEN1_ID)  ? {4'b1100, ien133, 1'b0, ien110} :
 (sfraddr == AMSK0_ID) ? amsk0_reg :
 (sfraddr == TSTAT_ID) ? {lni, 3'b000, tdn, tfnf, tstat11, 1'b0} :
 (sfraddr == AMSK1_ID) ? amsk1_reg :
 (sfraddr == RSTAT_ID) ? {ovr, rcabt, ae, crce, rdn, rfne, rstat11, 1'b0} :
 (sfraddr == RFIFO_ID) ? rfifo :
 (sfraddr == IPN1_ID)  ? {4'b1100, ipn133, 1'b0, ipn110} :
 8'h00 ;
endmodule
