// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description: CoreSDLC Transmit logic
//				instantiates the following sub-blocks:
//				CORESDLC_FIFO, CORESDLC_CRC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13654 $
// SVN $Date: 2010-07-08 15:49:19 -0700 (Thu, 08 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4"
//
// *********************************************************************/
`timescale 1 ns / 1 ps
module CORESDLC_TRANSMIT (
input clk, 
input nreset, 
input bce, 
input txc, 
input xtclk, 
input ct, 
input[1:0] mode, 
input[1:0] pl, 
input gfien, 
input[7:0] ifsreg, 
input ten, 
input tenset, 
input tfifowr, 
input[7:0] tfifo, 
output wire tfnf, 
output reg tdn, 
output wire[7:0] ifs, 
output wire txd, 
output reg den, 
input[7:0] dib, 
output wire[1:0] addra,
output wire[1:0] addrb,
output wire wea,
output wire enb,
output wire[7:0] doa
);
localparam[1:0] RT_MODE = 2'b01; 
localparam[7:0] BOF_FL = 8'b01111110; 
localparam[7:0] PRE_FL = 8'b01010101; 
localparam[2:0] SM_IDLE = 0; 
localparam[2:0] SM_IDLEFLAG = 1; 
localparam[2:0] SM_PREAMBLE = 2; 
localparam[2:0] SM_BOF = 3; 
localparam[2:0] SM_DATA = 4; 
localparam[2:0] SM_CRC = 5; 
localparam[2:0] SM_EOF = 6; 
reg fiford; 
wire[7:0] fifo_out; 
wire empty; 
wire full; 
wire stuff_in; 
reg[2:0] stuff_count; 
reg stuffing; 
reg crcshift; 
reg crcgen; 
reg crcset; 
wire crcout; 
wire crcmatch; 
reg[7:0] data; 
wire data_out; 
reg iforce; 
reg[2:0] sm_nxt; 
reg[2:0] sm; 
reg[2:0] bit_count; 
reg[2:0] byte_count; 
reg int_ten; 
reg[8:1] ifs_count; 
reg ifs_num; 
reg ifs_expired; 
reg txc_ff; 
reg txc_ff2; 
reg[2:0] tcount; 
reg tce; 
reg td; 
reg tdata; 
CORESDLC_FIFO u_fifo (
	.nreset(nreset),
	.clk(clk),
	.datai(tfifo),
	.rd(fiford),
	.wr(tfifowr),
	.clear(tenset),
	.datao(fifo_out),
	.empty(empty),
	.full(full),
	.dib(dib),
	.addra(addra),
	.addrb(addrb),
	.wea(wea),
	.enb(enb),
	.doa(doa)
);
CORESDLC_CRC u_crc (
	.clk(clk),
	.nreset(nreset),
	.ct(ct),
	.set(crcset),
	.gen(crcgen),
	.shift(crcshift),
	.datai(data_out),
	.datao(crcout),
	.match(crcmatch)
);
assign tfnf = ~full ;
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        stuff_count <= 3'b101 ; 
        tdata <= 1'b1 ; 
    end
    else
    begin
        if (tce == 1'b1)
        begin
            if ((sm == SM_DATA | sm == SM_CRC) & mode != RT_MODE & 
            stuffing == 1'b0 & stuff_in == 1'b1)
            begin
                stuff_count <= stuff_count - 1'b1 ; 
            end
            else
            begin
                stuff_count <= 3'b101 ; 
            end 
        end 
        if (stuffing == 1'b0)
        begin
            tdata <= stuff_in ; 
        end
        else
        begin
            tdata <= 1'b0 ; 
        end 
    end
end 
always @(stuff_count)
begin
    if (stuff_count == 3'b000)
    begin
        stuffing <= 1'b1 ; 
    end
    else
    begin
        stuffing <= 1'b0 ; 
    end 
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        data <= 8'h00;
    end
    else
    begin
        if (tce == 1'b1 & stuffing == 1'b0)
        begin
            if (sm == SM_IDLE | bit_count == 3'b000)
            begin
                case (sm)
                    SM_IDLE, SM_IDLEFLAG :
                    begin
                        if (empty == 1'b0 & ten == 1'b1 & ifs_expired == 1'b1)
                        begin
                            if (mode == RT_MODE)
                            begin
                                data <= fifo_out ; 
                            end
                            else
                            begin
                                if (pl == 2'b00)
                                begin
                                    data <= BOF_FL ; 
                                end
                                else
                                begin
                                    data <= PRE_FL ; 
                                end
                            end
                        end
                        else
                        begin
                            data <= BOF_FL ; 
                        end 
                    end
                    SM_PREAMBLE :
                    begin
                        if (byte_count == 3'b000)
                        begin
                            data <= BOF_FL ; 
                        end
                        else
                        begin
                            data <= PRE_FL ; 
                        end 
                    end
                    SM_CRC :
                    begin
                        data <= BOF_FL ; 
                    end
                    SM_EOF :
                    begin
                        data <= BOF_FL ; 
                    end
                    default :
                    begin
                        data <= fifo_out ; 
                    end
                endcase 
            end
            else
            begin
                data <= {1'b0, data[7:1]} ; 
            end 
        end 
    end
end 
assign data_out = data[0] ;
assign stuff_in = (sm == SM_CRC) ? crcout : data_out ;
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        int_ten <= 1'b0 ; 
    end
    else
    begin
        if (ten == 1'b0)
        begin
            int_ten <= 1'b0 ; 
        end
        else
        begin
            if (tce == 1'b1)
            begin
                int_ten <= ten ; 
            end 
        end 
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        sm <= SM_IDLE ; 
    end
    else
    begin
        if (tce == 1'b1 & stuffing == 1'b0)
        begin
            sm <= sm_nxt ; 
        end 
    end
end 
always @(sm or ten or int_ten or gfien or empty or mode or pl or bit_count or
         byte_count or ifs_expired)
begin
    case (sm)
        SM_IDLE :
        begin
            if (ten == 1'b1 & empty == 1'b0 & ifs_expired == 1'b1)
            begin
                if (mode == RT_MODE)
                begin
                    sm_nxt <= SM_DATA ; 
                end
                else
                begin
                    if (pl == 2'b00)
                    begin
                        sm_nxt <= SM_BOF ; 
                    end
                    else
                    begin
                        sm_nxt <= SM_PREAMBLE ; 
                    end
                end
            end
            else
            begin
                if (gfien == 1'b1)
                begin
                    sm_nxt <= SM_IDLEFLAG ; 
                end
                else
                begin
                    sm_nxt <= SM_IDLE ; 
                end 
            end 
        end
        SM_IDLEFLAG :
        begin
            if (ten == 1'b1 & bit_count == 3'b000 & empty == 1'b0 &
            ifs_expired == 1'b1)
            begin
                if (mode == RT_MODE)
                begin
                    sm_nxt <= SM_DATA ; 
                end
                else
                begin
                    if (pl == 2'b00)
                    begin
                        sm_nxt <= SM_BOF ; 
                    end
                    else
                    begin
                        sm_nxt <= SM_PREAMBLE ; 
                    end 
                end 
            end
            else
            begin
                if (gfien == 1'b1)
                begin
                    sm_nxt <= SM_IDLEFLAG ; 
                end
                else
                begin
                    sm_nxt <= SM_IDLE ; 
                end 
            end 
        end
        SM_PREAMBLE :
        begin
            if (int_ten == 1'b0)
            begin
                sm_nxt <= SM_IDLE ; 
            end
            else
            begin
                if (bit_count == 3'b000 & byte_count == 3'b000)
                begin
                    sm_nxt <= SM_BOF ; 
                end
                else
                begin
                    sm_nxt <= SM_PREAMBLE ; 
                end 
            end 
        end
        SM_BOF :
        begin
            if (int_ten == 1'b0)
            begin
                sm_nxt <= SM_IDLE ; 
            end
            else
            begin
                if (bit_count == 3'b000)
                begin
                    sm_nxt <= SM_DATA ; 
                end
                else
                begin
                    sm_nxt <= SM_BOF ; 
                end 
            end 
        end
        SM_DATA :
        begin
            if (int_ten == 1'b0)
            begin
                sm_nxt <= SM_IDLE ; 
            end
            else
            begin
                if (bit_count == 3'b000 & empty == 1'b1)
                begin
                    if (mode == RT_MODE)
                    begin
                        sm_nxt <= SM_IDLE ; 
                    end
                    else
                    begin
                        sm_nxt <= SM_CRC ; 
                    end
                end
                else
                begin
                    sm_nxt <= SM_DATA ; 
                end 
            end
        end
        SM_CRC :
        begin
            if (int_ten == 1'b0)
            begin
                sm_nxt <= SM_IDLE ; 
            end
            else
            begin
                if (bit_count == 3'b000 & byte_count == 3'b000)
                begin
                    sm_nxt <= SM_EOF ; 
                end
                else
                begin
                    sm_nxt <= SM_CRC ; 
                end 
            end 
        end
        default :
        begin
            if (int_ten == 1'b0)
            begin
                sm_nxt <= SM_IDLE ; 
            end
            else
            begin
                if (bit_count == 3'b000)
                begin
                    if (gfien == 1'b1)
                    begin
                        sm_nxt <= SM_IDLEFLAG ; 
                    end
                    else
                    begin
                        sm_nxt <= SM_IDLE ; 
                    end 
                end
                else
                begin
                    sm_nxt <= SM_EOF ; 
                end 
            end 
        end
    endcase 
end 
always @(sm or bit_count or tce or stuffing)
begin
    if (sm == SM_DATA & bit_count == 3'b111 & tce == 1'b1 & stuffing == 1'b0)
    begin
        fiford <= 1'b1 ; 
    end
    else
    begin
        fiford <= 1'b0 ; 
    end 
end 
always @(sm or tce or stuffing)
begin
    if (sm == SM_BOF)
    begin
        crcset <= 1'b1 ; 
    end
    else
    begin
        crcset <= 1'b0 ; 
    end 
    if (sm == SM_DATA & tce == 1'b1 & stuffing == 1'b0)
    begin
        crcgen <= 1'b1 ; 
    end
    else
    begin
        crcgen <= 1'b0 ; 
    end 
    if (sm == SM_CRC & tce == 1'b1 & stuffing == 1'b0)
    begin
        crcshift <= 1'b1 ; 
    end
    else
    begin
        crcshift <= 1'b0 ; 
    end 
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        den <= 1'b1 ; 
    end
    else
    begin
        if (tce == 1'b1)
        begin
            if ((sm == SM_IDLE | sm == SM_IDLEFLAG) & sm_nxt != SM_PREAMBLE &
            sm_nxt != SM_BOF & sm_nxt != SM_DATA)
            begin
                den <= 1'b1 ; 
            end
            else
            begin
                den <= 1'b0 ; 
            end 
        end
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        bit_count <= 3'b111 ; 
    end
    else
    begin
        if (sm == SM_IDLE)
        begin
            bit_count <= 3'b111 ; 
        end
        else
        begin
            if (tce == 1'b1 & stuffing == 1'b0)
            begin
                bit_count <= bit_count - 1'b1 ; 
            end 
        end 
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        byte_count <= 3'b111 ; 
    end
    else
    begin
        if (sm == SM_IDLE | sm == SM_IDLEFLAG)
        begin
            case (pl)
                2'b01 :
                begin
                    byte_count <= 3'b000 ; 
                end
                2'b10 :
                begin
                    byte_count <= 3'b011 ; 
                end
                2'b11 :
                begin
                    byte_count <= 3'b111 ; 
                end
                default :
                begin
                    byte_count <= 3'b111 ; 
                end
            endcase 
        end
        else if (sm == SM_DATA)
        begin
            case (ct)
                1'b0 :
                begin
                    byte_count <= 3'b001 ; 
                end
                default :
                begin
                    byte_count <= 3'b011 ; 
                end
            endcase 
        end
        else
        begin
            if (tce == 1'b1 & stuffing == 1'b0)
            begin
                if (bit_count == 3'b000)
                begin
                    byte_count <= byte_count - 1'b1 ; 
                end 
            end 
        end 
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ifs_count <= 8'b10000000 ; 
        ifs_num <= 1'b1 ; 
    end
    else
    begin
        if (tce == 1'b1)
        begin
            if ((sm != SM_IDLE & sm != SM_IDLEFLAG) | (ifs_expired == 1'b1))
            begin
                if (ifsreg[7:1] == 7'b0000000)
                begin
                    ifs_count <= {1'b1, ifsreg[7:1]} ; 
                end
                else
                begin
                    ifs_count <= {1'b0, ifsreg[7:1]} ; 
                end 
                ifs_num <= 1'b1 ; 
            end
            else
            begin
                if (ifs_count == 8'b00000001 & ifs_num == 1'b1)
                begin
                    if (ifsreg[7:1] == 7'b0000000)
                    begin
                        ifs_count <= {1'b1, ifsreg[7:1]} ; 
                    end
                    else
                    begin
                        ifs_count <= {1'b0, ifsreg[7:1]} ; 
                    end 
                    ifs_num <= 1'b0 ; 
                end
                else
                begin
                    ifs_count <= ifs_count - 1'b1 ;
                end 
            end 
        end
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        ifs_expired <= 1'b0 ; 
    end
    else
    begin
        if (({ifs_count, ifs_num}) == 9'b000000000)
        begin
            ifs_expired <= 1'b1 ; 
        end
        else
        begin
            if (sm != SM_IDLE & sm != SM_IDLEFLAG)
            begin
                ifs_expired <= 1'b0 ; 
            end 
        end 
    end
end 
assign ifs = {ifs_count[7:1], ifs_num} ;
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        tdn <= 1'b0 ; 
    end
    else
    begin
        if (tenset == 1'b1)
        begin
            tdn <= 1'b0 ; 
        end
        else if ((tce == 1'b1 & sm == SM_EOF & bit_count == 3'b000) |
                 (tce == 1'b1 & sm == SM_DATA & mode == RT_MODE &
                  bit_count == 3'b000 & empty == 1'b1 & stuffing == 1'b0))
        begin
            tdn <= 1'b1 ; 
        end 
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        iforce <= 1'b0 ; 
    end
    else
    begin
        if ((ten == 1'b0 & gfien == 1'b0) | (sm == SM_IDLE & gfien == 1'b0))
        begin
            iforce <= 1'b1 ; 
        end
        else
        begin
            iforce <= 1'b0 ; 
        end 
    end 
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        txc_ff <= 1'b0 ; 
        txc_ff2 <= 1'b0 ; 
    end
    else
    begin
        txc_ff <= txc ; 
        txc_ff2 <= txc_ff ; 
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        tcount <= 3'b111 ;
        tce <= 1'b0 ; 
    end
    else
    begin
        case (xtclk)
            1'b0 :
            begin
                if (bce == 1'b1)
                begin
                    if (tcount == 3'b000)
                    begin
                        tce <= 1'b1 ; 
                    end
                    else
                    begin
                        tce <= 1'b0 ; 
                    end 
                    tcount <= tcount - 1'b1 ; 
                end
                else
                begin
                    tce <= 1'b0 ; 
                end 
            end
            default :
            begin
                if (txc_ff2 == 1'b1 & txc_ff == 1'b0)
                begin
                    tce <= 1'b1 ; 
                end
                else
                begin
                    tce <= 1'b0 ; 
                end 
            end
        endcase
    end
end 
always @(posedge clk or negedge nreset)
begin
    if (nreset == 1'b0)
    begin
        td <= 1'b1 ; 
    end
    else
    begin
        if (xtclk == 1'b1)
        begin
            if (txc_ff2 == 1'b1 & txc_ff == 1'b0)
            begin
                if (iforce == 1'b1)
                begin
                    td <= 1'b1 ; 
                end
                else
                begin
                    td <= tdata ; 
                end 
            end 
        end
        else
        begin
            if (tce == 1'b1)
            begin
                if (iforce == 1'b1)
                begin
                    td <= 1'b1 ; 
                end
                else
                begin
                    if (tdata == 1'b0)
                    begin
                        td <= ~td ; 
                    end 
                end 
            end 
        end
    end
end 
assign txd = td ;
endmodule
