// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description:	User Test-bench for CoreSDLC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13726 $
// SVN $Date: 2010-07-13 16:00:44 -0700 (Tue, 13 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4"
//
// History:			4/02/03 - TFB created
//				7/02/08 - EP  added APB wrapper
// *********************************************************************/
`timescale 1ns / 1ps

module testbench;

// constants ...

// System Clock cycle (in ns)
parameter	SYS_CLK_CYCLE	= 50;

// Various Signals
reg				PRESETN1;
reg				PRESETN2;
reg				PCLK1;
reg				PCLK2;
wire			SDIO;
wire			TX_VALID1;
wire			RX_VALID1;
wire			RX_ERROR1;
wire			RX_DONE1;
wire			TX_VALID2;
wire			RX_VALID2;
wire			RX_ERROR2;
wire			RX_DONE2;
wire	[7:0]	PDIO1;
reg		[7:0]	PDO1;
wire	[7:0]	PDIO2;
reg		[7:0]	PDO2;
reg		[5:0]	PADDR1;
reg		[5:0]	PADDR2;
reg				PWRITE1;
reg				PWRITE2;
reg				PENABLE1;
reg				PENABLE2;
reg				PSEL1;
reg				PSEL2;

integer			cyc;
integer			simerrors;

// various strings
reg [8*79:1]	init_str_mem [0:11];
reg [8*79:1]	uline_str,copyright_str,tb_name_str,
				tb_ver_str,pound_str;

// initialize strings
initial
begin: init_strings
uline_str		=
"_____________________________________________________________________________";
pound_str		=
"#############################################################################";
copyright_str	=
"(c) Copyright 2010 Actel Corporation. All rights reserved.";
tb_name_str		=
"User Testbench for: CoreSDLC";
tb_ver_str		=
"Version: 3.0 08July10                                                     ";

// initialization of testbench string
init_str_mem[00]	= "";
init_str_mem[01]	= "";
init_str_mem[02]	= uline_str;
init_str_mem[03]	= "";
init_str_mem[04]	= copyright_str;
init_str_mem[05]	= "";
init_str_mem[06]	= tb_name_str;
init_str_mem[07]	= tb_ver_str;
init_str_mem[08]	= uline_str;
init_str_mem[09]	= "";
init_str_mem[10]	= "";
init_str_mem[11]	= "";

end

//---------------------------------------------------------------------
//-- constants for register map
//---------------------------------------------------------------------
//PA: Changed address mapping
localparam	GMOD_A		= 6'h00;
localparam	TFIFO_A		= 6'h04; 
localparam	PCON_A		= 6'h08; 
localparam	BAUD_A		= 6'h0C;
localparam	ADR0_A		= 6'h10;
localparam	IFS_A		= 6'h14;
localparam	ADR1_A		= 6'h18;
localparam	ADR2_A		= 6'h1c;              
localparam	ADR3_A		= 6'h20;
localparam	IEN1_A		= 6'h24;
localparam	AMSK0_A		= 6'h28;
localparam	TSTAT_A		= 6'h2C;
localparam	AMSK1_A		= 6'h30;
localparam	RSTAT_A		= 6'h34;
localparam	RFIFO_A		= 6'h38;
localparam	IPN1_A		= 6'h3c; 


// instantiate DUT(s):

// DUT component 1
USER_CORESDLC USER_CORESDLC_primary (
.PRESETN	(PRESETN1),
.PCLK		(PCLK1),
.PADDR  	(PADDR1),
.PWRITE		(PWRITE1),
.PENABLE	(PENABLE1),
.PSEL		(PSEL1),
.PDIO		(PDIO1),
.SDIO		(SDIO),
.TX_VALID	(TX_VALID1),
.RX_VALID	(RX_VALID1),
.RX_ERROR	(RX_ERROR1),
.RX_DONE 	(RX_DONE1)
);

// DUT component 2
USER_CORESDLC USER_CORESDLC_secondary (
.PRESETN	(PRESETN2),
.PCLK		(PCLK2),
.PADDR  	(PADDR2),
.PWRITE		(PWRITE2),
.PENABLE	(PENABLE2),
.PSEL		(PSEL2),
.PDIO		(PDIO2),
.SDIO		(SDIO),
.TX_VALID	(TX_VALID2),
.RX_VALID	(RX_VALID2),
.RX_ERROR	(RX_ERROR2),
.RX_DONE 	(RX_DONE2)
);

// System Clocks
initial cyc = SYS_CLK_CYCLE;
always #(cyc/2) PCLK1 = ~PCLK1;
// make PCLK1 and PCLK2 different (since they would be on separate boards anyway)
always @ (PCLK1) #10 PCLK2 <= PCLK1;

// emulate bidirectional data busses of CPU's
assign PDIO1 = (PWRITE1 & PSEL1 & PENABLE1) ? PDO1: {8{1'bz}};
assign PDIO2 = (PWRITE2 & PSEL2 & PENABLE2) ? PDO2: {8{1'bz}};

// Create one initial block for primary node, and one for secondary node
// (primary_stim will be the main stimulus)

initial
begin: primary_stim
integer i;

	$timeformat(-9, 2, " ns", 3);

	i		= 0;
	simerrors	= 0;

	// print out copyright info, testbench version, name of testbench, etc.
	for (i=0;i<12;i=i+1)
		$display("%0s",init_str_mem[i]);

	PRESETN1	= 0;
	PCLK1		= 0;
	PDO1		= 8'b0;
	PADDR1		= 6'b0;
	PWRITE1		= 0;
	PENABLE1	= 0;
	PSEL1		= 0;

	// synch to falling edge of clock
	@ (posedge PCLK1);
	@ (negedge PCLK1);

	#(cyc * 2);
	checksig(RX_VALID1,"RX_VALID1",0); 
	checksig(RX_ERROR1,"RX_ERROR1",0); 
	checksig(RX_DONE1,"RX_DONE1",0); 
	checksig(TX_VALID1,"TX_VALID1",0); 

	PRESETN1	= 1;
	#(cyc * 2);

	// read the various registers to verify reset values
	apb_read1(PCON_A,8'h00); 
	apb_read1(GMOD_A,8'h00);
	apb_read1(TFIFO_A,8'h00);
	apb_read1(BAUD_A,8'h00);
	apb_read1(ADR0_A,8'h00);
	// IFS counter contents will be read in progress of counting down,
	// thus, will not be 0x00
	apb_read1(IFS_A,8'hFD); //modified by EP 3/4/8 from 8'h01
	apb_read1(ADR1_A,8'h00);
	apb_read1(ADR2_A,8'h00);
	apb_read1(ADR3_A,8'h00);
	apb_read1(IEN1_A,8'hC0);
	apb_read1(AMSK0_A,8'h00);
	// Tx FIFO not full flag is '1' on reset
	apb_read1(TSTAT_A,8'h04);
	apb_read1(AMSK1_A,8'h00);
	apb_read1(RSTAT_A,8'h00);
	apb_read1(RFIFO_A,8'h00);
	apb_read1(IPN1_A,8'hC0);


	// initialize each CoreSDLC node

	// no idle flags
	apb_write1(PCON_A,8'h00);

	// fastest baud rate (clk freq./((0 + 1) * 8))
	apb_write1(BAUD_A,8'h00);

	// int. clock, normal mode, 8-bit addr.,16-bit CRC,8-bit preamble, 
	apb_write1(GMOD_A,8'h02);

	// 2 bit time Interframe Space value (fastest between frames)
	apb_write1(IFS_A,8'h02);

	// allow interrupt enables for Rx error flags, Rx FIFO not empty, and
	// Tx FIFO not full conditions
	apb_write1(IEN1_A,8'hFF);

	#(cyc * 2);
	checksig(TX_VALID1,"TX_VALID1",1); 

	// set address of primary node to 0xB1, secondary node to 0xD2
	// (set other adr1-3 to 0xFF to only match those for broadcast addr)
	apb_write1(ADR0_A,8'hB1);
	apb_write1(ADR1_A,8'hFF);
	apb_write1(ADR2_A,8'hFF);
	apb_write1(ADR3_A,8'hFF);


	// set Tx enable bit in primary and secondary nodes (the only writable bit
	// in TSTAT reg, which clears Tx done and Tx FIFO as well)
	apb_write1(TSTAT_A,8'h02);

	// set Rx enable bit in primary and secondary nodes (the only writable bit
	// in RSTAT reg, which clears Rx done, Rx error flags)
	apb_write1(RSTAT_A,8'h02);

	// wait for IFS (Inter Frame Space) counter to downcount ...
	#(cyc * 1024);
	apb_read1(IFS_A,8'h03);

	// write Tx data into primary node Tx FIFO -> secondary node Rx
	txframe1(40'hD212345678,5); 

	// wait until Rx done signal occurs on secondary node
	rxwait2;

	// wait for and process Rx frame (originated from secondary node Tx frame)
	rxframe1(88'hB1FEFDFCFBFAF9F8F7F6F5,11); 

	// wait until Rx done signal occurs on primary node
	rxwait1; 

	// check that the rdn (receive done) bit is set in the RSTAT reg
	checkrdn1; 


	#(cyc * 2);
	#(cyc * 1);
	#(cyc * 1);

	// print out number of simulation errors (if any) at end of sim
	$display("");
	$display("%0s",pound_str);
	$display("All Tests complete.");
	$display("%0s",pound_str);
	$display("");
	$display("%0s",uline_str); $display("");
	$display("Number of simulation mis-match errors encountered: %0d",
	simerrors); $display("");
	$display("Done with simulation.");
	$display("%0s",uline_str); $display(""); $display("");

	#(cyc * 1);
	$finish;

end

// initial block for secondary node

initial
begin: secondary_stim
integer i;
	i		= 0;

	PRESETN2	= 0;
	PCLK2		= 0;
	PDO2		= 8'b0;
	PADDR2		= 7'b0;
	PWRITE2		= 0;
	PENABLE2	= 0;
	PSEL2		= 0;

	// synch to falling edge of clock
	@ (posedge PCLK2);
	@ (negedge PCLK2);

	#(cyc * 2);
	checksig(RX_VALID2,"RX_VALID2",0); 
	checksig(RX_ERROR2,"RX_ERROR2",0); 
	checksig(RX_DONE2,"RX_DONE2",0); 
	checksig(TX_VALID2,"TX_VALID2",0); 

	PRESETN2		= 1;
	#(cyc * 2);

	// read the various registers to verify reset values
	apb_read2(PCON_A,8'h00);
	apb_read2(GMOD_A,8'h00);
	apb_read2(TFIFO_A,8'h00);
	apb_read2(BAUD_A,8'h00);
	apb_read2(ADR0_A,8'h00);
	// IFS counter contents will be read in progress of counting down,
	// thus, will not be 0x00
	apb_read2(IFS_A,8'hFD); //modified by EP 3/4/8 from 8'h01
	apb_read2(ADR1_A,8'h00);
	apb_read2(ADR2_A,8'h00);
	apb_read2(ADR3_A,8'h00);
	apb_read2(IEN1_A,8'hC0);
	apb_read2(AMSK0_A,8'h00);
	// Tx FIFO not full flag is '1' on reset
	apb_read2(TSTAT_A,8'h04);
	apb_read2(AMSK1_A,8'h00);
	apb_read2(RSTAT_A,8'h00);
	apb_read2(RFIFO_A,8'h00);
	apb_read2(IPN1_A,8'hC0);

	// initialize each CoreSDLC node

	// no idle flags
	apb_write2(PCON_A,8'h00);

	// fastest baud rate (clk freq./((0 + 1) * 8))
	apb_write2(BAUD_A,8'h00);

	// int. clock, normal mode, 8-bit addr.,16-bit CRC,8-bit preamble, 
	apb_write2(GMOD_A,8'h02);

	// 2 bit time Interframe Space value (fastest between frames)
	apb_write2(IFS_A,8'h02);

	// allow interrupt enables for Rx error flags, Rx FIFO not empty, and
	// Tx FIFO not full conditions
	apb_write2(IEN1_A,8'hFF);

	#(cyc * 2);
	checksig(TX_VALID2,"TX_VALID2",1);

	// set address of primary node to 0xB1, secondary node to 0xD2
	// (set other adr1-3 to 0xFF to only match those for broadcast addr)
	apb_write2(ADR0_A,8'hD2);
	apb_write2(ADR1_A,8'hFF);
	apb_write2(ADR2_A,8'hFF);
	apb_write2(ADR3_A,8'hFF);

	// set address mask of secondary node to 0x03 for multicast functionality
	// (all frames sent to addresses 0xD0 to 0xD3 will match)
	apb_write2(AMSK0_A,8'h03);

	// set Tx enable bit in primary and secondary nodes (the only writable bit
	// in TSTAT reg, which clears Tx done and Tx FIFO as well)
	apb_write2(TSTAT_A,8'h02);

	// set Rx enable bit in primary and secondary nodes (the only writable bit
	// in RSTAT reg, which clears Rx done, Rx error flags)
	apb_write2(RSTAT_A,8'h02);

	// wait for IFS (Inter Frame Space) counter to downcount ...
	#(cyc * 1024);
	apb_read2(IFS_A,8'h03);

	// wait for and process Rx frame (originated from primary node Tx frame)
	rxframe2(40'hD212345678,5);

	// wait until Rx done signal occurs on secondary node
	rxwait2;

	// check that the rdn (receive done) bit is set in the RSTAT reg
	checkrdn2;

	// write Tx data into secondary node Tx FIFO -> primary node Rx
	txframe2(88'hB1FEFDFCFBFAF9F8F7F6F5,11);

	// wait until Rx done signal occurs on primary node
	rxwait1;

	#(cyc * 1);

end


//----------------------------------------------------------------------
// tasks
//----------------------------------------------------------------------

// ----------- check the value of a signal or bus (up to 128 bits) ---------

task checksig;
input [127:0] d;
input [8*17:1] sig_name;
input [127:0] v;
begin
	$timeformat(-9, 2, " ns", 3);
	if (d != v)
	begin
		$display("");
		$display("%0s",pound_str);
		$display("ERROR!!! Mismatch on signal %0s",sig_name);
		$display("At time: %0t",$time);
		$display("Expected value was: 0x%0x, observed value is: 0x%0x",v,d);
		$display("%0s",pound_str);
		$display("");
		simerrors = simerrors + 1;
	end
end
endtask

// ***** These emulate an APB write *****
task apb_write1;
input [7:0] a;
input [7:0] d;
begin
  //$display("CPU Write %04x = %04x",a,d);
  @(negedge PCLK1);  
  PWRITE1  = 1'b0;
  @(negedge PCLK1);  
  PENABLE1  = 1'b0;
  PSEL1  = 1'b1;
  PADDR1 = a[6:0];
  PDO1  = d;  
  PWRITE1  = 1'b1;
  @(negedge PCLK1);  
  PENABLE1  = 1'b1;
  @(negedge PCLK1);  
  PWRITE1  = 1'b0;
  PENABLE1  = 1'b0;
  PSEL1  = 1'b0;
end
endtask

task apb_write2;
input [7:0] a;
input [7:0] d;
begin
//  $display("APB Write %04x = %04x",a,data);
  @(negedge PCLK2);  
  PWRITE2  = 1'b0;
  @(negedge PCLK2);  
  PENABLE2  = 1'b0;
  PSEL2  = 1'b1;
  PADDR2 = a[6:0];
  PDO2  = d;  
  PWRITE2  = 1'b1;
  @(negedge PCLK2);  
  PENABLE2  = 1'b1;
  @(negedge PCLK2);  
  PWRITE2  = 1'b0;
  PENABLE2  = 1'b0;
  PSEL2  = 1'b0;
end
endtask


// ***** APB READS *****
task apb_read1;
input [7:0] a;
input [7:0] d;
begin
  @(negedge PCLK1);  
  PWRITE1  = 1'b1;
  @(negedge PCLK1);  
  PENABLE1  = 1'b0;
  PSEL1  = 1'b1;
  PADDR1 = a[6:0];
  PWRITE1  = 1'b0;
  @(negedge PCLK1);  
  PENABLE1  = 1'b1;
  @(negedge PCLK1);
  PWRITE1  = 1'b1;
  PENABLE1  = 1'b0;
  PSEL1  = 1'b0;
  checksig(PDIO1,"APB Read Data Bus",d);
//  $display("APB Read %04x = %04x",address,data);
end
endtask

task apb_read2;
input [7:0] a;
input [7:0] d;
begin
  @(negedge PCLK2);  
  PWRITE2  = 1'b1;
  @(negedge PCLK2);  
  PENABLE2  = 1'b0;
  PSEL2  = 1'b1;
  PADDR2 = a[6:0];
  PWRITE2  = 1'b0;
  @(negedge PCLK2);  
  PENABLE2  = 1'b1;
  @(negedge PCLK2);
  PWRITE2  = 1'b1;
  PENABLE2  = 1'b0;
  PSEL2  = 1'b0;
//  $display("APB Read %04x = %04x",address,data);
  checksig(PDIO2,"APB Read Data Bus",d);
end
endtask

//---------------------------------------------------------------------
//-- Wait for SDLC tx frames
//---------------------------------------------------------------------
task txwait1;
begin
	while (!TX_VALID1) #(cyc * 1);
	#(cyc * 1);
end
endtask
task txwait2;
begin
	while (!TX_VALID2) #(cyc * 1);
	#(cyc * 1);
end
endtask

//---------------------------------------------------------------------
//-- Emulate CPU writing bytes to CoreSDLC for SDLC tx frames
//---------------------------------------------------------------------
task txframe1;
input	[(8*32)-1:0]	d;	// tx frame - make bigger if needed
input	[7:0]		bytes;	// size of tx frame (d) in bytes
integer		        i;
reg	[8*20:1]	ndstr;
integer			node;
reg	[7:0]		d1;
begin
	// using 0 for primary node, other numbers for secondary
	ndstr	= "Primary";
	node	= 0;
	$display("%0s",uline_str); $display("");
	$display("%0s node: %0d",ndstr,node);
	$display("Transmitting SDLC frame: 0x%0x",d);
	$display("At time: %0t.",$time);
	$display("");
	for (i=0;i<bytes;i=i+1) begin
		d1	= 8'hff&(d>>(((bytes-1)-i)*8));
		txwait1;
		apb_write1(TFIFO_A,d1);
		#(cyc * 1);
	end
end
endtask

task txframe2;
input	[(8*32)-1:0]	d;	// tx frame - make bigger if needed
input	[7:0]		bytes;	// size of tx frame (d) in bytes
integer			i;
reg	[8*20:1]	ndstr;
integer			node;
reg	[7:0]		d1;
begin
	// using 0 for primary node, other numbers for secondary
	ndstr	= "Secondary";
	node	= 1;
	$display("%0s",uline_str); $display("");
	$display("%0s node: %0d",ndstr,node);
	$display("Transmitting SDLC frame: 0x%0x",d);
	$display("At time: %0t.",$time);
	$display("");
	for (i=0;i<bytes;i=i+1) begin
		d1	= 8'hff&(d>>(((bytes-1)-i)*8));
		txwait2;
		apb_write2(TFIFO_A,d1);
		#(cyc * 1);

	end
end
endtask

//---------------------------------------------------------------------
//-- Wait for SDLC rx frames
//---------------------------------------------------------------------
task rxwait1;
begin
	while (!RX_DONE1) #(cyc * 1);
	#(cyc * 1);
end
endtask
task rxwait2;
begin
	while (!RX_DONE2) #(cyc * 1);
	#(cyc * 1);
end
endtask

//---------------------------------------------------------------------
//-- Emulate CPU reading bytes from CoreSDLC for SDLC rx frames
//---------------------------------------------------------------------
task rxframe1;
input	[(8*32)-1:0]	d;	// tx frame - make bigger if needed
input	[7:0]		bytes;	// size of tx frame (d) in bytes
integer			i;
reg	[8*20:1]	ndstr;
integer			node;
reg	[7:0]		d1;
begin
	// using 0 for primary node, other numbers for secondary
	ndstr	= "Primary";
	node	= 0;
	for (i=0;i<bytes;i=i+1) begin
		d1	= 8'hff&(d>>(((bytes-1)-i)*8));
		while (!RX_VALID1) #(cyc * 1);
		#(cyc * 1);
		apb_read1(RFIFO_A,d1);
		#(cyc * 2);
	end
	$display("%0s",uline_str); $display("");
	$display("%0s node: %0d",ndstr,node);
	$display("Received SDLC frame: 0x%0x",d);
	$display("At time: %0t.",$time);
	$display("");
end
endtask

task rxframe2;
input	[(8*32)-1:0]	d;	// tx frame - make bigger if needed
input	[7:0]		bytes;	// size of tx frame (d) in bytes
integer			i;
reg	[8*20:1]	ndstr;
integer			node;
reg	[7:0]		d1;
begin
	// using 0 for primary node, other numbers for secondary
	ndstr	= "Secondary";
	node	= 1;
	for (i=0;i<bytes;i=i+1) begin
		d1	= 8'hff&(d>>(((bytes-1)-i)*8));
		while (!RX_VALID2) #(cyc * 1);
		#(cyc * 1);
		apb_read2(RFIFO_A,d1);
		#(cyc * 2);
	end
	$display("%0s",uline_str); $display("");
	$display("%0s node: %0d",ndstr,node);
	$display("Received SDLC frame: 0x%0x",d);
	$display("At time: %0t.",$time);
	$display("");
end
endtask

//---------------------------------------------------------------------
//-- Check that the rdn (receive done) bit is set in the RSTAT reg after
//-- an SDLC frame is received in its entirety
//---------------------------------------------------------------------
task checkrdn1;
begin
	apb_read1(RSTAT_A,8'h08);
	// clear the rdn bit
	apb_write1(RSTAT_A,8'h02);
	#(cyc * 2);
	apb_read1(RSTAT_A,8'h02);
	#(cyc * 2);
end
endtask

task checkrdn2;
begin
	apb_read2(RSTAT_A,8'h08);
	// clear the rdn bit
	apb_write2(RSTAT_A,8'h02);
	#(cyc * 2);
	apb_read2(RSTAT_A,8'h02);
	#(cyc * 2);
end
endtask

endmodule

