// ********************************************************************/
// Actel Corporation Proprietary and Confidential
// Copyright 2010 Actel Corporation.  All rights reserved.
//
// ANY USE OR REDISTRIBUTION IN PART OR IN WHOLE MUST BE HANDLED IN
// ACCORDANCE WITH THE ACTEL LICENSE AGREEMENT AND MUST BE APPROVED
// IN ADVANCE IN WRITING.
//
// Description:	Example design incorporating CoreSDLCUser Test-bench for CoreSDLC
//
// Revision Information:
// Date			Description
// ----			-----------------------------------------
// 08July10		Production Release Version 3.0
//
// SVN Revision Information:
// SVN $Revision: 13680 $
// SVN $Date: 2010-07-09 13:12:23 -0700 (Fri, 09 Jul 2010) $
//
// Resolved SARs
// SAR      Date     Who   Description
//
// Notes:
// best viewed with tabstops set to "4"
//
// History:			4/02/03 - TFB created
//				7/02/08 - EP  added APB wrapper
// *********************************************************************/

`timescale 1ns / 1ps
module USER_CORESDLC (
    input PRESETN, 
    input PCLK, 
    input[5:0] PADDR, 
    input PENABLE, 
    input PSEL,
    input PWRITE, 
    inout wire[7:0] PDIO,
    inout wire SDIO, 
    output wire TX_VALID,
    output wire RX_VALID,
    output wire RX_ERROR,
    output wire RX_DONE
);
   

    // Control signals
    // Serial interface
    // Interrupts
    // Interrupt priorities
    // Special function register interface
    // Sfr data output
    //--------------------
    // internal signals
    //--------------------
    wire POE;
    wire SOE; 

    wire RXD; 
    wire TXC; 
    wire RXC; 
    wire TXD; 
    wire DEN; 
    wire TV; 
    wire RE; 
    wire RV; 
    wire RDN; 
    wire TDN;
    wire PTV; 
    wire PRE; 
    wire PRV; 

    wire[7:0] PWDATA; 
    wire[7:0] PRDATA; 
    wire PREADY;
    wire PSLVERR;

    // external clocks not used in this example design
    assign TXC = 1'b0 ;
    assign RXC = 1'b0 ;

    //-----------------------------------------
    // instantiate macro
    //-----------------------------------------
    CORESDLC CORESDLC_inst (
		.PCLK(PCLK),
		.PRESETN(PRESETN),
		.RXD(RXD),
		.TXC(TXC),
		.RXC(RXC),
		.TXD(TXD),
		.DEN(DEN),
		.TV(TV),
		.RE(RE),
		.RV(RV),
		.RDN(RDN),
		.TDN(TDN),
		.PTV(PTV),
		.PRE(PRE),
		.PRV(PRV),
		.PWDATA(PWDATA),
		.PADDR(PADDR),
		.PENABLE(PENABLE),
		.PWRITE(PWRITE),
		.PSEL(PSEL),
		.PRDATA(PRDATA),
		.PREADY(PREADY),
		.PSLVERR(PSLVERR)
	);
    //-----------------------------------------
    // control direction of APB data bus
    //-----------------------------------------
    assign POE = ~PWRITE & PSEL & PENABLE;
    assign PDIO = (POE == 1'b1) ? PRDATA : {8{1'bz}};
    assign PWDATA = PDIO ;

    //-----------------------------------------
    // control direction of serial SDLC data
    //-----------------------------------------
    assign SOE = ~DEN ;
    assign SDIO = (SOE == 1'b1) ? TXD : 1'bz ;
    assign RXD = SDIO ;

    assign TX_VALID = TV ;
    assign RX_VALID = RV ;
    assign RX_ERROR = RE ;
    assign RX_DONE = RDN ;

endmodule
